#!/usr/bin/perl -w
# $Id$
# Ian Beckwith <ianb@nessie.mcc.ac.uk>
#

use XML::PFIF;
package XML::PFIF::Note;
@ISA=qw(XML::PFIF);
use strict;

sub new
{
    my $proto=shift;
    my $class=ref($proto) || $proto;
    my $self={};
    bless($self,$class);

    my $hash=shift;
    
    @{$self->{fields}}=qw(note_record_id entry_date author_name author_email author_phone source_date found email_of_found_person phone_of_found_person last_known_location text);
    @{$self->{channelfields}}=qw(title description lastBuildDate link);

    for my $field (@{$self->{fields}})
    {
	if(defined($hash) && exists($hash->{$field}))
	{
	    $self->$field($hash->{$field}); 
	}
	else
	{
	    $self->$field('');
	}
    }

    return $self;
}

sub person
{
    my ($self,$newval)=(@_);
    if(defined($newval))
    {
	$self->{person}=$newval;
    }
    return $self->{person};
}


sub to_xml
{
    my $self=shift;
    my $str="    <pfif:note>\n";
    for my $field (@{$self->{fields}})
    {
	next if($self->{$field} eq '' && $self->found() eq "false" &&
		(($field eq "email_of_found_person") ||
		($field eq "phone_of_found_person")));
	if($self->{$field} eq '')
	{
	    $str .= "    <pfif:$field/>\n";
	}
	else
	{
	    $str .= "    <pfif:$field>$self->{$field}</pfif:$field>\n";
	}
    }
    $str .= "  </pfif:note>\n";
    return $str;
}




#
#  4.2. note records
#
#   Each note record belongs to exactly one person record. There may be
#   any number of note records associated with a particular person record.
#   (See below for implementation notes. A database might implement this
#   by including a foreign key, person_record_id, that refers to the
#   person record. An object-oriented representation might implement this
#   by embedding a list of note objects within the person object.)
#
#   Not being able to remove or update records was a huge problem with
#   September 11 survivor databases. note records resolve this problem
#   while avoiding the problem of synchronizing conflicting changes. Every
#   note has a timestamp and information on the author of the note.
#   Applications can use the timestamp to determine the most recent value
#   of a given field. Users can use the author information to ascertain
#   the reliabiliy of a given field.
#
#    Information About a Missing Person That Changes Over Time (11 fields)
#
#   The found, email_of_found_person, phone_of_found_person and
#   last_known_location fields store data that changes over time. When
#   these fields are present in a note record, the record is specifying
#   new values for these fields, and the source_date field indicates the
#   date that the new values took effect. So, for example, an application
#   that wants to display the most recent known location can look for the
#   note with the latest source_date that has a non-empty
#   last_known_location field.
#


#   note_record_id (string):
#          Unique identifier for this record, which consists of a domain
#          name followed by a slash and a local identifier. The domain
#          name identifies this record's home repository, which is the
#          authority for this record. The format of the local identifier
#          is up to the home repository. When the note_record_id begins
#          with a domain other than the application's own domain, it means
#          this record is a clone of a record from another source.

sub note_record_id
{
    my ($self,$newval)=(@_);
    if(defined($newval))
    {
	$self->{note_record_id}=$newval;
    }
    return $self->{note_record_id};
}


#   entry_date (string in the form "yyyy-mm-ddThh:mm:ssZ"):
#          Date in UTC that this record was created in its containing
#          repository (not the date of creation in the home repository).
#          The repository must guarantee that this value increases
#          monotonically for each newly added record, so that a client can
#          update a copy of a repository by querying for all records with
#          an entry_date greater than or equal to the entry_date of the
#          last received record.
sub entry_date
{
    my ($self,$newval)=(@_);
    if(defined($newval))
    {
	if($newval eq '' || $self->validate_date($newval))
	{
	    $self->{entry_date}=$newval;
	}
	else
	{
	    warn("entry_date: bad date format (must be 'yyyy-mm-ddThh:mm:ssZ'): $newval\n");
	    return undef;
	}
    }
    return $self->{entry_date};
}

#
#   author_name (string):
#          The full name of the person who entered this note.

sub author_name
{
    my ($self,$newval)=(@_);
    if(defined($newval))
    {
	$self->{author_name}=$newval;
    }
    return $self->{author_name};
}

#   author_email (string):
#          The preferred contact e-mail address of the person who entered
#          this note.
sub author_email
{
    my ($self,$newval)=(@_);
    if(defined($newval))
    {
	$self->{author_email}=$newval;
    }
    return $self->{author_email};
}


#   author_phone (string):
#          The preferred contact phone number of the person who entered
#          this note.
sub author_phone
{
    my ($self,$newval)=(@_);
    if(defined($newval))
    {
	$self->{author_phone}=$newval;
    }
    return $self->{author_phone};
}


#
#   source_date (string in the form "yyyy-mm-ddThh:mm:ssZ"):
#          The date in UTC that this note was originally created in its
#          home repository. In most cases, notes should be sorted by this
#          field for display.
sub source_date
{
    my ($self,$newval)=(@_);
    if(defined($newval))
    {
	if($newval eq '' || $self->validate_date($newval))
	{
	    $self->{source_date}=$newval;
	}
	else
	{
	    warn("source_date: bad date format (must be 'yyyy-mm-ddThh:mm:ssZ'): $newval\n");
	    return undef;
	}
    }
    return $self->{source_date};
}


#   found (boolean string, "true" or "false"):
#          This value is "true" if the missing person has been personally
#          contacted or seen, or "false" otherwise. The text field of this
#          note should describe HOW and WHEN the person was contacted or
#          seen.

sub found
{
    my ($self,$newval)=(@_);
    if(defined($newval))
    {
	if(($newval=~/true/i) ||
	   ($newval eq "1")   ||
	   ($newval=~/yes/i))
	{
	    $self->{found}="true";
	}
	elsif(($newval=~/false/i) ||
	      ($newval eq "0")    ||
	      ($newval=~/no/i))
	{
	    $self->{found}="false";
	}
	elsif($newval eq '')
	{
	    $self->{found}='';
	}
	else
	{
	    warn("found: bad value: $newval: expected \"true\" or \"false\"\n");
	    return undef;
	}
    }
    return $self->{found};
}


#   email_of_found_person (string):
#          The preferred contact e-mail address of the FOUND person. This
#          field is present ONLY if the person has been FOUND. The text
#          field of this note should describe HOW the person's contact
#          information was determined.
sub email_of_found_person
{
    my ($self,$newval)=(@_);
    if(defined($newval))
    {
	$self->{email_of_found_person}=$newval;
    }
    return $self->{email_of_found_person};
}

#
#   phone_of_found_person (string):
#          The preferred contact phone number of the FOUND person. This
#          field is present ONLY if the person has been FOUND. The text
#          field of this note should describe HOW the person's contact
#          information was determined.
sub phone_of_found_person
{
    my ($self,$newval)=(@_);
    if(defined($newval))
    {
	$self->{phone_of_found_person}=$newval;
    }
    return $self->{phone_of_found_person};
}

#
#   last_known_location (string):
#          A free-form description of the last known location of the
#          person being sought, including the city, state, and as much
#          detail as possible. The text field of this note should describe
#          HOW the person's location was determined.
sub last_known_location
{
    my ($self,$newval)=(@_);
    if(defined($newval))
    {
	$self->{last_known_location}=$newval;
    }
    return $self->{last_known_location};
}

#
#   text (large string):
#          Free-form text description of the person's current condition,
#          situation and location details, where they were last seen,
#          corrections to other information, etc.
#
sub text
{
    my ($self,$newval)=(@_);
    if(defined($newval))
    {
	$self->{text}=$newval;
    }
    return $self->{text};
}



sub to_rss
{
    my($self)=@_;
    my $person=$self->person();
    my $str="    <item>\n";
    my $indent="    ";
    $str .= $person->to_xml();
    $str .= $indent . $self->tag("guid",$self->note_record_id());
    $str .= $indent . $self->tag("author",$self->author_email() ." (". $self->author_name() .")");
    $str .= $indent . $self->tag("pubDate",$self->date_to_rfc822($person->source_date()));

    $str .= $indent . "<description>\n";
    $str .= "<p>Record entered at ". $self->date_to_human($self->source_date()) ."</p>\n";
    $str .= "<p>Note entered by ". $self->author_name ." (" .$self->author_email() .
	") at " . $self->date_to_human($self->source_date()) .":\n";
    $str .= "<pre>\n";
    $str .= $self->text() . "\n";
    if($self->found() eq 'true')
    {
	$str .= "* Person has been found.\n";
    }
    if($self->email_of_found_person() ne '')
    {
	$str .= "* E-mail address of found person: ". $self->email_of_found_person() ."\n";
    }
    if($self->phone_of_found_person() ne '')
    {
	$str .= "* Phone number of found person: ". $self->phone_of_found_person() ."\n";
    }
    if($self->last_known_location() ne '')
    {
	$str .= "* Last known location: " . $self->last_known_location() ."\n";
    }
    $str .= "</pre>\n";
	    
    $str .= $indent . "</description>\n";
    $str .= "    </item>\n";
    return $str;
}

1;

		
__END__

