#!/usr/bin/perl -w
# $Id$
# Ian Beckwith <ianb@nessie.mcc.ac.uk>
#

use XML::PFIF;
use XML::PFIF::Note;

package XML::PFIF::Person;
@ISA=qw(XML::PFIF);

use strict;

#  3.1. person records
#
#   A person record contains 17 fields. There may be multiple person
#   records for the same person. In fact, any given application that
#   imports data from multiple sources is likely to have multiple person
#   records for the same person. It is up to the application to associate
#   the records (see the Database Implementation section below). It is
#   recommended that applications keep copies of all the records, and
#   separately keep track of which records correspond to the same person.
#
#    Static Tracking Information About the Record Itself (8 fields)
#
#   Meta-information like this is essential because it allows people to
#   trace and ascertain the reliability of the data they are looking at,
#   which was a big problem with survivor databases for September 11.
#

sub new
{
    my $proto=shift;
    my $class=ref($proto) || $proto;
    my $self={};
    bless($self,$class);

    my $hash=shift;
    
    $self->{notes}=();
    @{$self->{fields}}=qw(person_record_id entry_date author_name author_email author_phone source_name source_date source_url first_name last_name home_city home_state home_neighborhood home_street home_zip photo_url other);
    @{$self->{channelfields}}=qw(title description lastBuildDate link);

    for my $field (@{$self->{fields}})
    {
	if(defined($hash) && exists($hash->{$field}))
	{
	    $self->$field($hash->{$field}); 
	}
	else
	{
	    $self->$field('');
	}
    }
    return $self;
}

sub notes
{
    my $self=shift;
    return @{$self->{notes}};
}
    
sub to_xml
{
    my $self=shift;
    my $str="  <pfif:person>\n";
    for my $field (@{$self->{fields}})
    {
	if($self->{$field} eq '')
	{
	    $str .= "    <pfif:$field/>\n";
	}
	else
	{
	    $str .= "    <pfif:$field>$self->{$field}</pfif:$field>\n";
	}
    }
    #FIXME: sort by date 
    for my $note (@{$self->{notes}})
    {
	$str .= $note->to_xml();
    }
    $str .= "  </pfif:person>\n";

    return $str;
}

sub add_note
{
    my ($self,@args)=@_;
    my $note=new XML::PFIF::Note(@args);
    $note->person($self);
    push(@{$self->{notes}},$note);
    return $note;
}


#    Static Tracking Information About the Record Itself (8 fields)

# person_record_id (string)
#          Unique identifier for the record, which consists of a domain
#          name followed by a slash and a local identifier. Each
#          application is assumed to reside at or belong to a particular
#          domain. When the person_record_id begins with the application's
#          own domain, it means this application is the authority for this
#          record, and the format of the local identifier is up to the
#          application. When the person_record_id begins with some other
#          domain, it means this record is a clone of a record from
#          another source, and the other domain is the authority for this
#          record.
#

sub person_record_id
{
    my ($self,$newval)=(@_);
    if(defined($newval))
    {
	$self->{person_record_id}=$newval;
    }
    return $self->{person_record_id};
}

#   entry_date (string in the form "yyyy-mm-ddThh:mm:ssZ"):
#          Date that this record was created in the local application, in
#          UTC. The application must guarantee that this value increases
#          monotonically so that a client can update a copy of a database
#          by querying for all records with an entry_date greater than the
#          entry_date of the last received record.
sub entry_date
{
    my ($self,$newval)=(@_);
    if(defined($newval))
    {
	if($newval eq '' || $self->validate_date($newval))
	{
	    $self->{entry_date}=$newval;
	}
	else
	{
	    warn("entry_date: bad date format (must be 'yyyy-mm-ddThh:mm:ssZ'): $newval\n");
	    return undef;
	}
    }
    return $self->{entry_date};
}

#   author_name (string):
#          The full name of the person who entered this record.
sub author_name
{
    my ($self,$newval)=(@_);
    if(defined($newval))
    {
	$self->{author_name}=$newval;
    }
    return $self->{author_name};
}

#   author_email (string):
#          The preferred contact e-mail address of the person who entered
#          this record.
sub author_email
{
    my ($self,$newval)=(@_);
    if(defined($newval))
    {
	$self->{author_email}=$newval;
    }
    return $self->{author_email};
}

#   author_phone (string):
#          The preferred contact phone number of the person who entered
#          this record.
sub author_phone
{
    my ($self,$newval)=(@_);
    if(defined($newval))
    {
	$self->{author_phone}=$newval;
    }
    return $self->{author_phone};
}

#   source_name (string):
#          If this data record is a clone of a record from an external
#          source, this field is present and contains the name of that
#          external source.
sub source_name
{
    my ($self,$newval)=(@_);
    if(defined($newval))
    {
	$self->{source_name}=$newval;
    }
    return $self->{source_name};
}


#   source_date (string in the form "yyyy-mm-ddThh:mm:ssZ"):
#          If this data record is a clone of a record from an external
#          source, this field is present and contains the date in UTC that
#          this record was originally posted to that source location.
sub source_date
{
    my ($self,$newval)=(@_);
    if(defined($newval))
    {
	if($newval eq '' || $self->validate_date($newval))
	{
	    $self->{source_date}=$newval;
	}
	else
	{
	    warn("source_date: bad date format (must be 'yyyy-mm-ddThh:mm:ssZ'): $newval\n");
	    return undef;
	}
    }
    return $self->{source_date};
}

#   source_url (string):
#          If this data record is a clone of a record from an external
#          source, this field is present and contains the URL to that
#          source record (as specific as possible, down to the URL of the
#          individual record).
sub source_url
{
    my ($self,$newval)=(@_);
    if(defined($newval))
    {
	$self->{source_url}=$newval;
    }
    return $self->{source_url};
}


#    Static Identifying Information About a Missing Person (9 fields)
#
#   These fields are specifically for identifying the person and should be
#   for data that never changes. These are the fields to search on.
#   Insisting on all capitals and no accents is ugly, but it makes
#   searches more likely to converge on the correct record. The other
#   field is a very crude way to import foreign data, but the formatting
#   guidelines should make it possible to extract the data again if there
#   is a desperate need to do so. For other, free-form text was chosen
#   instead of XML to make it easy for an application to display other
#   directly in the UI.
#
#   first_name (string, all capitals, no accents):
#          First name of the person sought or found, optionally followed
#          by a space and any middle names or middle initials.

sub first_name
{
    my ($self,$newval)=(@_);
    if(defined($newval))
    {
	$self->{first_name}=$self->fixup_string($newval);
    }
    return $self->{first_name};
}

#   last_name (string, all capitals, no accents):
#          Last name of the person sought or found.
sub last_name
{
    my ($self,$newval)=(@_);
    if(defined($newval))
    {
	$self->{last_name}=$self->fixup_string($newval);
    }
    return $self->{last_name};
}

#   home_city (string, city name, all capitals, no accents):
#          Home city of the person sought or found.
sub home_city
{
    my ($self,$newval)=(@_);
    if(defined($newval))
    {
	$self->{home_city}=$self->fixup_string($newval);
    }
    return $self->{home_city};
}

#   home_state (string, two-letter postal abbreviation):
#          Home state of the person sought or found.
sub home_state
{
    my ($self,$newval)=(@_);
    if(defined($newval))
    {
	if($newval ne '')
	{
	    # FIXME: check against list of valid states
	    if($newval !~ /^[a-zA-z][a-zA-z]$/)
	    {
		warn("home_state: $newval: not a valid 2-letter state code\n");
		return undef;
	    }
	    $newval =~ tr/[a-z]/[A-Z]/;
	}
	$self->{home_state}=$newval;
    }
    return $self->{home_state};
}

#   home_neighborhood (string, all capitals, no accents):
#          Name of the home neighborhood of the person sought or found.
sub home_neighborhood
{
    my ($self,$newval)=(@_);
    if(defined($newval))
    {
	$self->{home_neighborhood}=$self->fixup_string($newval);
    }
    return $self->{home_neighborhood};
}


#   home_street (string, all capitals, no accents):
#          Street name (no number) of the home address of the person
#          sought or found.
sub home_street
{
    my ($self,$newval)=(@_);
    if(defined($newval))
    {
	$self->{home_street}=$self->fixup_string($newval);
	# remove numbers also
	$self->{home_street}=~s/\d//g;
    }
    return $self->{home_street};
}

#   home_zip (integer):
#          Zip code of the home address of the person sought or found.
sub home_zip
{
    my ($self,$newval)=(@_);
    if(defined($newval))
    {
	if($newval eq '')
	{
	    $self->{home_zip}='';
	}
	elsif($newval =~ /^\s*(\d+)\s*$/)
	{
	    $self->{home_zip}=$1;
	}
	else
	{
	    warn("home_zip: $newval: not a valid zip code\n");
	    return undef;
	}
    }
    return $self->{home_zip};
}

#   photo_url (string):
#          URL to an image of an identifying photograph of the person
#          sought or found.
sub photo_url
{
    my ($self,$newval)=(@_);
    if(defined($newval))
    {
	$self->{photo_url}=$newval;
    }
    return $self->{photo_url};
}

#   other (large string):
#          Free-form text containing any other static data fields brought
#          in from other sources. (Non-static data imported from other
#          sources should go into a note record.) Short fields should be
#          on a single line with the field name, a colon, and the field
#          value. Long fields can be given as a line with the field name
#          and a colon, then text indented on the following lines. Field
#          names for data imported from other applications should begin
#          with the domain name and a slash. For example, if a birthdate
#          is imported from an ICRC record, it might look like this:
#
#icrc.org/birthdate: 1976-02-26
#
#          A description of the person in free-form text can also go here,
#          with the field name "description". For example:
#
#description:
#    Dark hair, in her late thirties.
#    Also goes by the names "Kate" or "Katie".

sub other
{
    my ($self,$newval)=(@_);
    if(defined($newval))
    {
	$self->{other}=$newval;
    }
    return $self->{other};
}

sub add_other
{
    my ($self,$newkey,$newval)=(@_);
    if(defined($newkey) && defined($newval))
    {
	$self->{other} .= "$newkey:";
	if( (length($newkey)+length($newval)) > 80 )
	{
	    $self->{other} .= "\n";
	}
	else
	{
	    $self->{other} .= " ";
	}

	$self->{other} .= $newval;
    }
    return $self->{other};
}


sub to_rss
{
    my($self)=@_;
    my $str="    <item>\n      <pfif:pfif>\n";
    my $indent="    ";
    $str .= $self->to_xml();
    $str .="      </pfif:pfif>\n";
    $str .= $indent . $self->tag("guid",$self->person_record_id());
    $str .= $indent . $self->tag("title",$self->first_name() ." ". $self->last_name());
    $str .= $indent . $self->tag("author",$self->author_email() ." (". $self->author_name() .")");
    $str .= $indent . $self->tag("pubDate",$self->date_to_rfc822($self->source_date()));

    $str .= $indent . "<description>\n";
    $str .= "<p>Person sought:</p>\n";
    if($self->photo_url() ne '')
    {
	$str .="<p><img src=\"". $self->photo_url() . "\" /></p>\n";
    }
    $str .= "<pre>\n";
    $str .= $self->first_name() . " " . $self->last_name() . "\n";
    $str .= $self->home_street() ."\n";
    $str .= $self->home_neighborhood() . "\n";
    $str .= $self->home_city() . ", " . $self->home_state ." ".
	    $self->home_zip ."\n\n";
    $str .= $self->other();
    $str .= "</pre>\n";
    $str .= "<p>Record entered by:</p>\n";
    $str .= "<pre>\n";
    $str .= $self->author_name() ."\n";
    $str .= $self->author_email() ."\n";
    $str .= $self->author_phone() ."\n";
    $str .= "</pre>\n";
    $str .= "<p>Record entered at ". $self->date_to_human($self->source_date()) . ".</p>\n";


    for my $note ($self->notes())
    {
	$str .= "<p>Note entered by ". $note->author_name ." (" .$note->author_email() .
	    ") at " . $self->date_to_human($note->source_date()) .":</p>\n";
	$str .= "<pre>\n";
	$str .= $note->text() . "\n";
	if($note->found() eq 'true')
	{
	    $str .= "* Person has been found.\n";
	}
	if($note->email_of_found_person() ne '')
	{
	    $str .= "* E-mail address of found person: ". $note->email_of_found_person() ."\n";
	}
	if($note->phone_of_found_person() ne '')
	{
	    $str .= "* Phone number of found person: ". $note->phone_of_found_person() ."\n";
	}
	if($note->last_known_location() ne '')
	{
	    $str .= "* Last known location: " . $note->last_known_location() ."\n";
	}
	$str .= "</pre>\n";
    }

    $str .= $indent . "</description>\n";
    $str .= $indent . $self->tag("source",$self->source_name());
    $str .= $indent . $self->tag("link",$self->source_url());
    $str .= "    </item>\n";
    return $str;
}

1;

__END__

